package javax.management.remote;

import com.sun.jmx.remote.util.EnvHelp;
import java.util.StringTokenizer;
import java.io.IOException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.net.UnknownHostException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import com.sun.jmx.remote.util.ClassLogger;
import java.util.BitSet;
import java.io.Serializable;

public class JMXServiceURL implements Serializable
{
    private static final long serialVersionUID = 8173364409860779292L;
    private static final String INVALID_INSTANCE_MSG = "Trying to deserialize an invalid instance of JMXServiceURL";
    private static final Exception randomException;
    private static final BitSet alphaBitSet;
    private static final BitSet numericBitSet;
    private static final BitSet alphaNumericBitSet;
    private static final BitSet protocolBitSet;
    private static final BitSet hostNameBitSet;
    private String protocol;
    private String host;
    private int port;
    private String urlPath;
    private transient String toString;
    private static final ClassLogger logger;
    
    public JMXServiceURL(String s) throws MalformedURLException {
        int length = s.length();
        int i = 0;
        while (i < length) {
            int char1 = s.charAt(i);
            if (char1 < ' ' || char1 >= '\u007f') {
                throw new MalformedURLException("Service URL contains non-ASCII character 0x" + Integer.toHexString(char1));
            }
            ++i;
        }
        int length2 = "service:jmx:".length();
        if (!s.regionMatches(true, 0, "service:jmx:", 0, length2)) {
            throw new MalformedURLException("Service URL must start with service:jmx:");
        }
        int index = indexOf(s, ':', length2);
        this.protocol = s.substring(length2, index).toLowerCase();
        if (!s.regionMatches(index, "://", 0, 3)) {
            throw new MalformedURLException("Missing \"://\" after protocol name");
        }
        int fromIndex = index + 3;
        int indexOfFirstNotInSet;
        if (fromIndex < length && s.charAt(fromIndex) == '[') {
            indexOfFirstNotInSet = s.indexOf(93, fromIndex) + 1;
            if (indexOfFirstNotInSet == 0) {
                throw new MalformedURLException("Bad host name: [ without ]");
            }
            this.host = s.substring(fromIndex + 1, indexOfFirstNotInSet - 1);
            if (!isNumericIPv6Address(this.host)) {
                throw new MalformedURLException("Address inside [...] must be numeric IPv6 address");
            }
        }
        else {
            indexOfFirstNotInSet = indexOfFirstNotInSet(s, JMXServiceURL.hostNameBitSet, fromIndex);
            this.host = s.substring(fromIndex, indexOfFirstNotInSet);
        }
        int indexOfFirstNotInSet2;
        if (indexOfFirstNotInSet < length && s.charAt(indexOfFirstNotInSet) == ':') {
            if (this.host.length() == 0) {
                throw new MalformedURLException("Cannot give port number without host name");
            }
            int beginIndex = indexOfFirstNotInSet + 1;
            indexOfFirstNotInSet2 = indexOfFirstNotInSet(s, JMXServiceURL.numericBitSet, beginIndex);
            String substring = s.substring(beginIndex, indexOfFirstNotInSet2);
            try {
                this.port = Integer.parseInt(substring);
            }
            catch (NumberFormatException obj) {
                throw new MalformedURLException("Bad port number: \"" + substring + "\": " + obj);
            }
        }
        else {
            indexOfFirstNotInSet2 = indexOfFirstNotInSet;
            this.port = 0;
        }
        int beginIndex2 = indexOfFirstNotInSet2;
        if (beginIndex2 < length) {
            this.urlPath = s.substring(beginIndex2);
        }
        else {
            this.urlPath = "";
        }
        this.validate();
        return;
    }
    
    public JMXServiceURL(String s, String s2, int n) throws MalformedURLException {
        this(s, s2, n, null);
    }
    
    public JMXServiceURL(String s, String s2, int port, String urlPath) throws MalformedURLException {
        if (s == null) {
            s = "jmxmp";
        }
        if (s2 == null) {
            InetAddress localHost;
            try {
                localHost = InetAddress.getLocalHost();
            }
            catch (UnknownHostException obj) {
                throw new MalformedURLException("Local host name unknown: " + obj);
            }
            s2 = localHost.getHostName();
            try {
                validateHost(s2, port);
            }
            catch (MalformedURLException t) {
                if (JMXServiceURL.logger.fineOn()) {
                    JMXServiceURL.logger.fine("JMXServiceURL", "Replacing illegal local host name " + s2 + " with numeric IP address (see RFC 1034)", t);
                }
                s2 = localHost.getHostAddress();
            }
        }
        if (s2.startsWith("[")) {
            if (!s2.endsWith("]")) {
                throw new MalformedURLException("Host starts with [ but does not end with ]");
            }
            s2 = s2.substring(1, s2.length() - 1);
            if (!isNumericIPv6Address(s2)) {
                throw new MalformedURLException("Address inside [...] must be numeric IPv6 address");
            }
            if (s2.startsWith("[")) {
                throw new MalformedURLException("More than one [[...]]");
            }
        }
        this.protocol = s.toLowerCase();
        this.host = s2;
        this.port = port;
        if (urlPath == null) {
            urlPath = "";
        }
        this.urlPath = urlPath;
        this.validate();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        String s = (String)fields.get("host", null);
        int value = fields.get("port", -1);
        String protocol = (String)fields.get("protocol", null);
        String urlPath = (String)fields.get("urlPath", null);
        if (protocol == null || urlPath == null || s == null) {
            StringBuilder append = new StringBuilder("Trying to deserialize an invalid instance of JMXServiceURL").append('[');
            int n = 1;
            if (protocol == null) {
                append.append("protocol=null");
                n = 0;
            }
            if (s == null) {
                append.append((n != 0) ? "" : ",").append("host=null");
                n = 0;
            }
            if (urlPath == null) {
                append.append((n != 0) ? "" : ",").append("urlPath=null");
            }
            append.append(']');
            throw new InvalidObjectException(append.toString());
        }
        else {
            if (s.contains("[") || s.contains("]")) {
                throw new InvalidObjectException("Invalid host name: " + s);
            }
            try {
                this.validate(protocol, s, value, urlPath);
                this.protocol = protocol;
                this.host = s;
                this.port = value;
                this.urlPath = urlPath;
            }
            catch (MalformedURLException ex) {
                throw new InvalidObjectException("Trying to deserialize an invalid instance of JMXServiceURL: " + ex.getMessage());
            }
            return;
        }
    }
    
    private void validate(String str, String s, int i, String str2) throws MalformedURLException {
        int indexOfFirstNotInSet = indexOfFirstNotInSet(str, JMXServiceURL.protocolBitSet, 0);
        if (indexOfFirstNotInSet == 0 || indexOfFirstNotInSet < str.length() || !JMXServiceURL.alphaBitSet.get((int)str.charAt(0))) {
            throw new MalformedURLException("Missing or invalid protocol name: \"" + str + "\"");
        }
        validateHost(s, i);
        if (i < 0) {
            throw new MalformedURLException("Bad port: " + i);
        }
        if (str2.length() > 0 && !str2.startsWith("/") && !str2.startsWith(";")) {
            throw new MalformedURLException("Bad URL path: " + str2);
        }
        return;
    }
    
    private void validate() throws MalformedURLException {
        this.validate(this.protocol, this.host, this.port, this.urlPath);
    }
    
    private static void validateHost(String s, int n) throws MalformedURLException {
        if (s.length() == 0) {
            if (n != 0) {
                throw new MalformedURLException("Cannot give port number without host name");
            }
            return;
        }
        else {
            if (isNumericIPv6Address(s)) {
                try {
                    InetAddress.getByName(s);
                }
                catch (Exception cause) {
                    MalformedURLException throwable = new MalformedURLException("Bad IPv6 address: " + s);
                    EnvHelp.initCause(throwable, cause);
                    throw throwable;
                }
            }
            else {
                int length = s.length();
                int n2 = 46;
                boolean n3 = false;
                int bitIndex = 0;
                for (int i = 0; i < length; ++i) {
                    int char1 = s.charAt(i);
                    int value = JMXServiceURL.alphaNumericBitSet.get(char1);
                    if (n2 == 46) {
                        bitIndex = char1;
                    }
                    if (!(value == 0)) {
                        n2 = 97;
                    }
                    else if (char1 == '-') {
                        if (n2 == 46) {
                            break;
                        }
                        n2 = 45;
                    }
                    else if (char1 == '.') {
                        n3 = true;
                        if (n2 != 97) {
                            break;
                        }
                        n2 = 46;
                    }
                    else {
                        n2 = 46;
                        break;
                    }
                }
                try {
                    if (n2 != 97) {
                        throw JMXServiceURL.randomException;
                    }
                    if (n3 && !JMXServiceURL.alphaBitSet.get(bitIndex)) {
                        StringTokenizer o = new StringTokenizer(s, ".", true);
                        int j = 0;
                        while (j < 4) {
                            int int1 = Integer.parseInt(o.nextToken());
                            if (int1 < 0 || int1 > 255) {
                                throw JMXServiceURL.randomException;
                            }
                            if (j < 3 && !o.nextToken().equals(".")) {
                                throw JMXServiceURL.randomException;
                            }
                            ++j;
                        }
                        if (o.hasMoreTokens()) {
                            throw JMXServiceURL.randomException;
                        }
                    }
                }
                catch (Exception ex) {
                    throw new MalformedURLException("Bad host: \"" + s + "\"");
                }
            }
            return;
        }
    }
    
    public String getProtocol() {
        return this.protocol;
    }
    
    public String getHost() {
        return this.host;
    }
    
    public int getPort() {
        return this.port;
    }
    
    public String getURLPath() {
        return this.urlPath;
    }
    
    @Override
    public String toString() {
        if (this.toString != null) {
            return this.toString;
        }
        StringBuilder sb = new StringBuilder("service:jmx:");
        sb.append(this.getProtocol()).append("://");
        String host = this.getHost();
        if (isNumericIPv6Address(host)) {
            sb.append('[').append(host).append(']');
        }
        else {
            sb.append(host);
        }
        int port = this.getPort();
        if (port != 0) {
            sb.append(':').append(port);
        }
        sb.append(this.getURLPath());
        return this.toString = sb.toString();
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof JMXServiceURL)) {
            return false;
        }
        JMXServiceURL jmxServiceURL = (JMXServiceURL)o;
        return jmxServiceURL.getProtocol().equalsIgnoreCase(this.getProtocol()) && jmxServiceURL.getHost().equalsIgnoreCase(this.getHost()) && jmxServiceURL.getPort() == this.getPort() && jmxServiceURL.getURLPath().equals(this.getURLPath());
    }
    
    @Override
    public int hashCode() {
        return this.toString().hashCode();
    }
    
    private static boolean isNumericIPv6Address(String s) {
        return s.indexOf(58) >= 0;
    }
    
    private static int indexOf(String s, char ch, int fromIndex) {
        int index = s.indexOf((int)ch, fromIndex);
        if (index < 0) {
            return s.length();
        }
        return index;
    }
    
    private static int indexOfFirstNotInSet(String s, BitSet set, int i) {
        int length = s.length();
        while (i < length) {
            int char1 = s.charAt(i);
            if (char1 < '\u0080' && set.get(char1)) {
                ++i;
                continue;
            }
            return i;
        }
        return i;
    }
    
    static {
        randomException = new Exception();
        alphaBitSet = new BitSet(128);
        numericBitSet = new BitSet(128);
        alphaNumericBitSet = new BitSet(128);
        protocolBitSet = new BitSet(128);
        hostNameBitSet = new BitSet(128);
        for (int i = 48; i <= 57; i = (char)(i + 1)) {
            JMXServiceURL.numericBitSet.set(i);
        }
        for (int j = 65; j <= 90; j = (char)(j + 1)) {
            JMXServiceURL.alphaBitSet.set(j);
        }
        for (int k = 97; k <= 122; k = (char)(k + 1)) {
            JMXServiceURL.alphaBitSet.set(k);
        }
        JMXServiceURL.alphaNumericBitSet.or(JMXServiceURL.alphaBitSet);
        JMXServiceURL.alphaNumericBitSet.or(JMXServiceURL.numericBitSet);
        JMXServiceURL.protocolBitSet.or(JMXServiceURL.alphaNumericBitSet);
        JMXServiceURL.protocolBitSet.set(43);
        JMXServiceURL.protocolBitSet.set(45);
        JMXServiceURL.hostNameBitSet.or(JMXServiceURL.alphaNumericBitSet);
        JMXServiceURL.hostNameBitSet.set(45);
        JMXServiceURL.hostNameBitSet.set(46);
        logger = new ClassLogger("javax.management.remote.misc", "JMXServiceURL");
    }
}
